
namespace eval findBu {
  set freq $eval_freq    
# number of timesteps between two updates; this number is smaller than $RESTART_freq
  set Max_outside 1   
# used for effiecient determination of crossing events
}

proc calc_findB { args } {; # procedure to determine if the system is within the current cell and to obtain the key kinetic information
    global ms_id; global ABORT_ON_CROSSING; global flag_self; global FWD_filename; global BWD_filename; global ID; global STAGE; global RESTART_freq;   global MAX_NUM_STEPS
    global stepnum; global incubation_start; global aborting; global backward_failed

    upvar findBu::freq freq
    upvar findBu::Max_outside Max_outside

#   id_list is -1 if still inside cell $cell_1
    set id_list -1

    if { [ expr {($stepnum % $freq) != 0}] } { incr stepnum; return $id_list}

    if { ([expr "$stepnum % $RESTART_freq"] == 0) && ($aborting == "False") } {
	if {$STAGE == "backward"} {
		append_to_file $BWD_filename "job$ID $stepnum"
	} else {
		append_to_file $FWD_filename "job$ID $stepnum"
	}
    }
    set N [llength $args]
    set count_Max_outside 0
    set cell_1 [lindex $ms_id 0]
    set cell_2 [lindex $ms_id 1]
    set ori_rmsd [lindex $args $cell_1]

    for {set j 1 } { $j < 2 } { incr j } { ; # Note here $j < 2 is only for neighbor milestones implementation
	if {$count_Max_outside == $Max_outside} {break}
	set now_id [expr ($cell_1 + $j)%$N]
	if { $now_id < $N} {
		if {$ori_rmsd >= [lindex $args $now_id]} {
			if {$flag_self!="True" && $now_id==$cell_2} {
				puts "cross self but will continue run"
				lset ms_id 0 $now_id
				lset ms_id 1 $cell_1
				incr count_Max_outside
			} else {
				set id_list $now_id
				incr count_Max_outside
			}
		}
	}
	if {$count_Max_outside == $Max_outside} {break}
	set now_id [expr ($cell_1 - $j)%$N]
	if { $now_id > -1} {
		if {$ori_rmsd >= [lindex $args $now_id]} {
			if {$flag_self!="True" && $now_id==$cell_2} {
				puts "cross self but will continue run"
				lset ms_id 0 $now_id
				lset ms_id 1 $cell_1
				incr count_Max_outside
			} else {
				set id_list $now_id
				incr count_Max_outside
			}
		}
	}
    }

    if {$id_list != -1} {
	if { $aborting != "True" } {
		if {$stepnum==0} {
			puts "findB colvar: ERROR!! the system is not within the cell ($cell_1) in the zeroth time step! (now it is in cell $id_list)"
			puts "Backward run failed."
			append_to_file $BWD_filename "job$ID failed"
			set backward_failed "True"
		}
		if { $STAGE == "backward"} { 
			if {$id_list!=$cell_2} {
				puts "Backward run succeeded."
				append_to_file $BWD_filename "job$ID succeeded"
				set backward_failed "False"
			} else {
				puts "Backward run failed."
				append_to_file $BWD_filename "job$ID failed"
				set backward_failed "True"
			}
		} else {
			puts "Forward run succeeded."
			append_to_file $FWD_filename "job$ID succeeded in crossing: $cell_1 $id_list , stepnum: $stepnum , incubation_time: [expr $stepnum - $incubation_start]"
		}
		if { ($ABORT_ON_CROSSING == True) } {
			set aborting "True"
		} else {
			lset ms_id 0 $id_list
			lset ms_id 1 $cell_1
			set incubation_start $stepnum
		}
	}
    }
    if {($stepnum > $MAX_NUM_STEPS) && ($MAX_NUM_STEPS != 0)} {
	set aborting "True"
    }
    incr stepnum
    return $id_list

}

proc calc_findB_gradient { args } {
   set grad {}
   foreach x $args {
	lappend grad 0
   }
return $grad
}

#define the colvar
set config "
colvarsTrajFrequency $DCD_freq

colvar {
   name                  fB

   width                 1
   lowerboundary        -10
   upperboundary         1000
   lowerwallconstant     10.0
   upperwallconstant     10.0

   scriptedFunction      findB
"

for {set i 0} {$i < $replicanum} {incr i} {
  append config " rmsd {
	atoms {
	atomsFile ./example/targcons_$i.pdb
	atomsCol B
	atomsColValue 1
	centerReference yes
	rotateReference yes
	refPositionsGroup {
	atomsFile ./example/refNVT_$i.pdb
	atomsCol B
	atomsColValue 1
	}
	refPositionsFile ./example/refNVT_$i.pdb
	refPositionsCol B
	refPositionsColValue 1
	}
	refPositionsFile ./example/targcons_$i.pdb
	refPositionsCol B
	refPositionsColValue 1
     }
   "
}
append config "}"

proc append_to_file { filename value } {
  set outfile [open $filename a]
  puts $outfile $value
  close $outfile
}


